package bookExamples.ch26Graphics.draw2d.shapes;

import bookExamples.ch26Graphics.draw2d.shapes.DJShape;
import bookExamples.ch26Graphics.draw2d.Containment;
import bookExamples.ch26Graphics.draw2d.Vec2d;
import bookExamples.ch26Graphics.draw2d.CircleFcn;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;

public class Hexagon extends DJShape implements PropertyEditor, Containment
{
    private String value = "Hexagon";
    private int radius = 1;
    int x1 = 0;
    int y1 = 0;
    Vec2d center = new Vec2d(0, 0);

    public Hexagon( int x1, int y1, int x2, int y2 )
    {
        super( x1, y1, x2, y2 );
        this.x1 =  x1;
        this.y1 =  y1;
        int dx = x1 - x2;
        int dy = y1 - y2;
        radius = (int) (Math.sqrt( dx * dx + dy * dy ) / 2.0);

        if ( x1 > x2 )
            this.x1 =  x2 ;
        if ( y1 > y2 )
            this.y1 =  y1;

        setCenter();
    }

    public DJShape getInstance( int x1, int y1, int x2, int y2 )
    {
        return new Hexagon( x1, y1, x2, y2 );
    }

    //modified the method of tracking the center point
    public Point getCenter()
    {
        return new Point((int) center.v[0], (int) center.v[1]);
    }

    private void setCenter() {
        center.v[0] = x1 + radius;
        center.v[1] = y1 + radius;
    } // end setCenter

    public void move(double x, double y)
    {
        center = new Vec2d(x, y);
    }

    public void setValue( Object value )
    {
        this.value = (String) value;
    }

    public Object getValue()
    {
        return value;
    }

    public boolean isPaintable()
    {
        return false;
    }

    public void paintValue( Graphics gfx, Rectangle box )
    {
        gfx.setClip( box );
        gfx.drawString( "Click to edit...", box.x + 5, box.y + 15 );
    }

    public String getJavaInitializationString()
    {
        return null;
    }

    public String getAsText()
    {
        return null;
    }

    public void setAsText( String text ) throws java.lang.IllegalArgumentException
    {
    }

    public String[] getTags()
    {
        return null;
    }

    public Component getCustomEditor()
    {
        JPanel jp = new JPanel( new FlowLayout() );

        jp.add( getRunAffinePanel() );

        return jp;
    }

    public boolean supportsCustomEditor()
    {
        return true;
    }

    public void addPropertyChangeListener( PropertyChangeListener listener )
    {
        getPcs().addPropertyChangeListener( listener );
    }

    public void removePropertyChangeListener( PropertyChangeListener listener )
    {
        getPcs().removePropertyChangeListener( listener );
    }

    private GeneralPath getGeneralPath()
    {
        CircleFcn circle = new CircleFcn( getX1(), getY1(), radius );
        double initialRotation = .75;
        GeneralPath gp = new GeneralPath();

        for ( double t = 0; t < 1; t = t + 0.167 )
        {
            Point2D point = circle.getPoint( t + initialRotation );
            if ( t == 0 ) gp.moveTo( (float) point.getX(), (float) point.getY() );
            if ( t > 0 ) gp.lineTo( (float) point.getX(), (float) point.getY() );
        }
        gp.closePath();

        return gp;
    }

    public Shape getShape()
    {
        Point pc = getCenter();
        x1 = pc.x - radius;
        y1 = pc.y - radius;

        double hexPointSpace = 1.0 / 6.0;

        CircleFcn cf = new CircleFcn((int) center.v[0], (int) center.v[1], radius);
        double initialRotation = 0.75;

        Point2D p1 = cf.getPoint(initialRotation + ( 0 * hexPointSpace) );
        Point2D p2 = cf.getPoint(initialRotation + ( 1 * hexPointSpace) );
        Point2D p3 = cf.getPoint(initialRotation + ( 2 * hexPointSpace) );
        Point2D p4 = cf.getPoint(initialRotation + ( 3 * hexPointSpace) );
        Point2D p5 = cf.getPoint(initialRotation + ( 4 * hexPointSpace) );
        Point2D p6 = cf.getPoint(initialRotation + ( 5 * hexPointSpace) );
        return getShape(p1, p2, p3, p4, p5, p6);
    }

    public Shape getShape(Point2D p1, Point2D p2, Point2D p3, Point2D p4, Point2D p5, Point2D p6) {
        GeneralPath gp = new GeneralPath();
        gp.moveTo((float) p1.getX(), (float) p1.getY());
        gp.lineTo((float) p2.getX(), (float) p2.getY());
        gp.lineTo((float) p3.getX(), (float) p3.getY());
        gp.lineTo((float) p4.getX(), (float) p4.getY());
        gp.lineTo((float) p5.getX(), (float) p5.getY());
        gp.lineTo( (float) p6.getX(), (float) p6.getY() );
        gp.closePath();
        AffineTransform at = getAffineTransform();
        return at.createTransformedShape(gp);
    }

    public boolean contains( double x, double y )
    {
        Shape s = getShape();
        return s.contains( x, y );
    }

    public void draw( Graphics g )
    {
        Graphics2D g2d = (Graphics2D) g;
        AffineTransform old = g2d.getTransform();
        AffineTransform affineTransform = getAffineTransform();
        g2d.transform( affineTransform );
        Color saveColor = g2d.getColor();
        g2d.setColor( getForeground() );

        Shape shape = getShape();
        g2d.draw( shape );
        drawAnnotation(g2d, old, saveColor,shape);
    }

    private void drawAnnotation(Graphics2D g2d, AffineTransform old, Color saveColor,Shape s)
    {
        double x1 = s.getBounds2D().getCenterX();
        double y1 = s.getBounds2D().getCenterY();
        setX1((int)x1);
        setY1((int)y1);
        g2d.drawString("(" + x1 + ", " + y1 + ")", getX1() + 3, getY1() + 3);
        g2d.setTransform(old);

        g2d.setColor(saveColor);
        restoreState(g2d);
    }
}
