package bookExamples.ch26Graphics.draw2d.shapes;


import bookExamples.ch26Graphics.draw2d.Containment;
import ip.transforms.Polygons;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import java.awt.geom.GeneralPath;

import gui.run.RunButton;
import gui.In;

/**
 * Modified by Gino Lopes
 * July 24, 2006
 * CR311/ECE430
 * Final Part A
 */

public class Lens2d extends DJShape
        implements PropertyEditor, Containment {
    private int w = 1;
    private int h = 1;
    private int xc = 0;
    private int yc = 0;
    int r1, r2;
    Polygon p = new Polygon();
    
    public DJShape getInstance(int x1, int y1, int x2, int y2) {
        return new Lens2d(x1, y1, x2, y2);
    }
    
    public Lens2d(int _x1, int _y1, int _x2, int _y2) {
        super(_x1, _y1, _x2, _y2);
        setX1(_x1);
        setY1(_y1);
        w = Math.abs(_x2 - this.getX1());
        h = Math.abs(_y2 - this.getY1());
        if (_x1 > _x2) setX1(_x2);
        if (_y1 > _y2) setY1(_y2);
        xc = this.getX1() + w / 2;
        yc = this.getY1() + h / 2;
        
        getPolygon();
    }
    
    public Point getCenter() {
        return new Point(xc - w / 4, yc);
    }
    
    public Polygon getPolygon(){
        //Polygon p = new Polygon();
        double v = (0.25);
        for (double i = (1 - v); i > (1 - (v + 0.5)); i -= 0.001) {
            Point2D p2 = getPoint(i);
            p.addPoint((int) p2.getX(), (int) p2.getY());
        } // end for loop
        return p;
    } // end getShape
    
    public Point2D getPoint(double t) {
        r1 = w / 2;
        r2 = h / 2;
        t = 2 * Math.PI * t;
        float x = (float) (r1 * Math.cos(t) + xc);
        float y = (float) (r2 * Math.sin(t) + yc);
        return new Point2D.Float(x, y);
    } // end getPoint
    
    public void draw(Graphics g) {
        Point2D pc = getCenter();
        int x = (int) pc.getX();
        int y = (int) pc.getY();
        Graphics2D g2d = (Graphics2D) g;
        AffineTransform saveAt = g2d.getTransform();
        AffineTransform affineTransform = getAffineTransform();
        
        // save the default color
        Color saveColor = g2d.getColor();
        // sets the color of the shape
        g2d.setColor(getForeground());
        
        g2d.transform(affineTransform);
        
        g2d.drawPolygon(p);
        
        // displays center2d point of cone
        g2d.fillOval(x, y, 2, 2);
        // displays the location of the center2d xy coordinates
        g2d.drawString("(" + x + "," + y + ")", x + 3, y + 3);
        
        g2d.setTransform(saveAt);
        // restores the color of the shape to the default color
        g2d.setColor(saveColor);
        // refreshes the drawn shape to the current state
        restoreState(g2d);
    }
    
    public boolean isPaintable() {
        //return true;
        return false;
    }
    
    public boolean supportsCustomEditor() {
        return true;
    }
    
    public Component getCustomEditor() {
        JPanel jp = new JPanel();
        jp.setLayout(new FlowLayout());
        jp.add(new RunButton("setForeground") {
            public void run() {
                Lens2d.this.setForeground(In.getColor());
                getPcs().firePropertyChange("lens", Lens2d.this, null);
            }
        });
        jp.add(getRunAffinePanel());
        return jp;
    }
    
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        getPcs().addPropertyChangeListener(listener);
    }
    
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        getPcs().removePropertyChangeListener(listener);
    }
    
    public Object getValue() {
        //return this;
        return null;
    }
    
    public void setValue(Object value) {
    }
    
    public String getAsText() {
        return null;
    }
    
    public String getJavaInitializationString() {
        return null;
    }
    
    public String[] getTags() {
        return new String[0];
    }
    
    public void setAsText(String text) throws IllegalArgumentException {
    }
    
    public void paintValue(Graphics gfx, Rectangle box) {
    }
    
    public boolean contains(double x, double y) {
        Shape s = getShape();
        return s.contains(x,y);
    }
    
    public Shape getShape() {
        AffineTransform at = getAffineTransform();
        return at.createTransformedShape(p);
    }
}
