package bookExamples.ch26Graphics.draw2d.shapes;

import bookExamples.ch26Graphics.draw2d.CircleFcn;
import bookExamples.ch26Graphics.draw2d.Containment;
import bookExamples.ch26Graphics.draw2d.Vec2d;
import gui.In;
import gui.run.RunButton;
import gui.run.RunSlider;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;

/**
 * Created by IntelliJ IDEA.
 * User: John Jacocks
 * Date: Dec 1, 2005
 * Time: 5:11:04 PM
 * Note:
 * Modifying class Pentagon by Dave Pavlik
 * to add interfaces for (?Intersects, Movable, &?) Containment
 * for 'prequel' to Final for ECE430, Fall 2005
 * using Triangle2d (by Enrique iturralde, Doug Lyon) as template
 */
public class Pentagon extends DJShape
        implements PropertyEditor, Containment {
    // possibly? Intersects, Movable,
    private String value = "Pentagon";

    int x1 = 0;
    int y1 = 0;
    int diameter = 1;
    int radius = 1;
    Vec2d center = new Vec2d(0, 0);
    double radiusSq;
    Shape s;

    public Pentagon(int _x1, int _y1, int _x2, int _y2) {
        super(_x1, _y1, _x2, _y2);
        x1 = _x1;
        y1 = _y1;
        int dx = x1 - _x2;
        int dy = y1 - _y2;
        diameter = (int) Math.sqrt(dx * dx + dy * dy);
        if (_x1 > _x2) x1 = _x2;
        if (_y1 > _y2) y1 = _y2;
        radius = diameter / 2;
        radiusSq = radius * radius;
        center.v[0] = x1 + radius;
        center.v[1] = y1 + radius;
    }

    public Pentagon(int _x1, int _y1, int _diameter) {
        super(_x1, _y1, _x1 + _diameter, _y1 + _diameter);
        x1 = _x1;
        y1 = _y1;
        diameter = _diameter;
        radius = diameter / 2;
    }

    public DJShape getInstance(int x1, int y1, int x2, int y2) {
        return new Pentagon(x1, y1, x2, y2);
    }


    public Point getCenter() {
        return new Point((int) center.v[0], (int) center.v[1]);
    }

    public void setCenter() {
        int radius = diameter / 2;
        center.v[0] = x1 + radius;
        center.v[1] = y1 + radius;
    }

    public Component getCustomEditor() {
        JPanel jp = new JPanel();
        jp.setLayout(new FlowLayout());
        jp.add(new JLabel("diameter"));
        jp.add(new RunSlider(RunSlider.HORIZONTAL, 10, 2000, diameter) {
            public void run() {
                diameter = getValue();
                getPcs().firePropertyChange("Circle2d", Pentagon.this, null);
            }
        });
        jp.add(new RunButton("setForeground") {
            public void run() {
                Pentagon.this.setForeground(In.getColor());

                getPcs().firePropertyChange("size", Pentagon.this, null);
            }
        });
        jp.add(getRunAffinePanel());

        jp.setLayout(new FlowLayout());

        return jp;
    }

    public void setValue(Object value) {
        this.value = (String) value;
    }

    public Object getValue() {
        return value;
    }

    public boolean isPaintable() {
        return false;
    }

    public void paintValue(Graphics gfx, Rectangle box) {
        gfx.setClip(box);
        gfx.drawString("Click to edit...", box.x + 5, box.y + 15);
    }

    public String getJavaInitializationString() {
        return null;
    }

    public String getAsText() {
        return null;
    }

    public void setAsText(String text) throws java.lang.IllegalArgumentException {
    }

    public String[] getTags() {
        return null;
    }

    public boolean supportsCustomEditor() {
        return true;
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        getPcs().addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        getPcs().removePropertyChangeListener(listener);
    }


    private GeneralPath getGeneralPath() {
        CircleFcn circle = new CircleFcn(getX1(), getY1(), radius);
        double initialRotation = .75;
        GeneralPath gp = new GeneralPath();

        for (double t = 0; t < 1; t += 0.2) {
            Point2D point = circle.getPoint(t + initialRotation);
            if (t == 0) gp.moveTo((float) point.getX(), (float) point.getY());
            if (t > 0) gp.lineTo((float) point.getX(), (float) point.getY());
        }
        gp.closePath();

        return gp;
    }

    public void draw(Graphics g) {
        Point pc = getCenter();
        Graphics2D g2d = (Graphics2D) g;
        AffineTransform old = g2d.getTransform();
        AffineTransform affineTransform = getAffineTransform();
        g2d.transform(affineTransform);
        Color saveColor = g2d.getColor();
        g2d.setColor(getForeground());

        Shape shape = getShape();
        g2d.draw(shape);
        drawAnnotation(g2d, old, saveColor,shape);
    }

    private void drawAnnotation(Graphics2D g2d, AffineTransform old, Color saveColor,Shape s) {
        //   */
        // g2d.fillOval(getX1(), getY1(), 2, 2);
        double x1 = s.getBounds2D().getCenterX();
        double y1 = s.getBounds2D().getCenterY();
        setX1((int)x1);
        setY1((int)y1);
        g2d.drawString("(" + x1 + ", " + x1 + ")", getX1() + 3, getY1() + 3);
        g2d.setTransform(old);

        g2d.setColor(saveColor);
        restoreState(g2d);
    }

    public String toString() {
        Point pc = getCenter();
        return
                "xc,yc,r=" + pc.x + "," + pc.y + "," + diameter;
    }

    public void move(double x, double y) {
        center = new Vec2d(x, y);
    }

    public boolean contains(double x, double y) {
        Shape s = getShape();
        return s.contains(x, y);
    }

    public Shape getShape() {
        Point pc = getCenter();
        x1 = pc.x - radius;
        y1 = pc.y - radius;

        int xc = (int) center.v[0];
        int yc = (int) center.v[1];
        CircleFcn cf = new CircleFcn(xc, yc, radius);
        double v = (0.25 + 1 / 5.0);
        Point2D p1 = cf.getPoint(1 - v);
        Point2D p2 = cf.getPoint(1 - 0.25);
        Point2D p3 = cf.getPoint(1 - (v + 3 / 5.0));
        Point2D p4 = cf.getPoint(1 - (v + 2 / 5.0));
        Point2D p5 = cf.getPoint(1 - (v + 1 / 5.0));
        return getShape(p1, p2, p3, p4, p5);
    }

    public Shape getShape(Point2D p1, Point2D p2, Point2D p3, Point2D p4, Point2D p5) {
        GeneralPath gp = new GeneralPath();
        gp.moveTo((float) p1.getX(), (float) p1.getY());
        gp.lineTo((float) p2.getX(), (float) p2.getY());
        gp.lineTo((float) p3.getX(), (float) p3.getY());
        gp.lineTo((float) p4.getX(), (float) p4.getY());
        gp.lineTo((float) p5.getX(), (float) p5.getY());
        gp.closePath();
        AffineTransform at = getAffineTransform();
        return at.createTransformedShape(gp);
    }
}
