package j2d;/*
* Copyright (c) 2006, Your Corporation. All Rights Reserved.
*/

import javax.media.jai.PlanarImage;
import javax.media.jai.RasterFactory;
import javax.media.jai.TiledImage;
import java.awt.*;
import java.awt.image.DataBuffer;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;


/**
 * This class demonstrates an implementation of a basic Template Matching
 * algorithm, using Raster(s) to access the data. This class inherits from
 * ImageProcessingTask.
 * IMPORTANT: This implementation does not lock access to the data while the
 * template matching is being done.
 */
public class TemplateMatching {
    // The input image, input template and output images.
    private PlanarImage input;
    private PlanarImage template;
    private TiledImage output;
    // Dimensions on the image and mask.
    private int imageWidth, imageHeight, templateWidth, templateHeight;




     public static Image getSpatialCorrelation(Image inputImage, Image templateImage){
          PlanarImage input = ImageUtils.getPlanarImage(inputImage);
        PlanarImage template = ImageUtils.getPlanarImage(templateImage);
        TemplateMatching tm = new TemplateMatching(input,template);
        tm.run();
        TiledImage output = tm.getOutput();
        return ImageUtils.getImage(output);
     }
    public static void main(String[] args) {
        PlanarImage input = ImageUtils.getPlanarImage(
                ImageUtils.getImage());
        PlanarImage template = ImageUtils.getPlanarImage(
                ImageUtils.getImage());
        TemplateMatching tm = new TemplateMatching(input,template);
        tm.run();
        ImageUtils.displayImage(ImageUtils.getImage(tm.getOutput()),"output");
    }
    public TemplateMatching(PlanarImage input, PlanarImage template) {
        this.input = input;
        this.template = template;
        // Get some info about the input and template images.
        imageWidth = input.getWidth();
        imageHeight = input.getHeight();
        templateWidth = template.getWidth();
        templateHeight = template.getHeight();

        // Create an output image with the same features as the input one.
        output = new TiledImage(0, 0, imageWidth, imageHeight, 0, 0,
                input.getSampleModel(), input.getColorModel());

    }




    /**
     * This method is the main driver of the algorithm, it will perform the template
     * matching.
     */
    public void run() {
        // Get the raster from the input image.
        Raster inputRaster = input.getData();
        // Get the raster from the template image.
        Raster templateRaster = template.getData();
        // Create arrays for the template and image region (same size as template).
        final int n = templateWidth * templateHeight;
        int[] pixelT = new int[n*3];
        int[] pixelI = new int[n*3];
        // Get the template pixels.
        System.out.println("n="+n);
        templateRaster.getPixels(0, 0, templateWidth, templateHeight, pixelT);
        // Create a WritableRaster.
        WritableRaster raster =
                RasterFactory.createBandedRaster(DataBuffer.TYPE_BYTE,
                        imageWidth, imageHeight, 1, new Point(0, 0));
        // Define some useful values.
        // Scan all pixels corresponding to the output image.
        for (int yi = templateHeight; yi < imageHeight - templateHeight; yi++) {
            for (int xi = templateWidth; xi < imageWidth - templateWidth; xi++) {
                int result = 0;
                // Get the region of the image under the moving window.
                inputRaster.getPixels(xi - templateWidth, yi - templateHeight, templateWidth, templateHeight, pixelI);
                // Get the sum of the absolute difference between the pixels on the image
                // and on the template. We can use a better-performing loop here.
                for (int t = 0; t < n; t++)
                    result += Math.abs(pixelT[t] - pixelI[t]);
                result = result / n;   //why divide by w*h?
                raster.setSample(xi, yi, 0, result);
            }
            // Set the output raster on the output image. Do it every line, so we can
            // get some partial results.
            output.setData(raster);

        }

    }


    /**
     * This method returns the output image.
     * @return the template matching resulting image.
     */
    public TiledImage getOutput() {
        return output;
    }


  }
