/*
 * Copyright (c) 2005 Your Corporation. All Rights Reserved.
 */
package j2d.transforms;

import futils.Futil;
import gui.ClosableJFrame;
import j2d.widgets.ScrollingAwtImagePanel;
import j2d.ImageUtils;

import javax.media.jai.*;
import j2d.widgets.ScrollingImagePanel;
import javax.media.jai.operator.DFTDescriptor;
import java.awt.*;
import java.awt.image.DataBuffer;
import java.awt.image.RenderedImage;
import java.awt.image.SampleModel;
import java.awt.image.renderable.ParameterBlock;

public class DftExample extends ClosableJFrame {
    public static void main(String[] args) {
        testDft();
    }

    private static void testDft2() {
        new DftExample(
                Futil.getReadFile("select an image").getAbsolutePath(),
                Futil.getReadFile("select an image").getAbsolutePath());
    }

    DftExample(String fileName1, String fileName2) {
        // Load the source image.
        PlanarImage src = JAI.create("fileload", fileName1);
        PlanarImage orig = JAI.create("fileload", fileName2);
        RenderedOp origDft = DFTImage(orig);
        RenderedOp gaussDft = DFTImage(src);
        RenderedOp conv = multComplex(origDft, gaussDft);

        PlanarImage idftimg = inverseDFTImage(conv);
        display(idftimg);

    }

    private PlanarImage getIdft(PlanarImage src) {
        PlanarImage idft = JAI.create("idft", src);

        // Create a RenderingHints object with desirable layout.
        ImageLayout il = new ImageLayout();
        il.setSampleModel(src.getSampleModel());
        il.setColorModel(src.getColorModel());
        RenderingHints rh = new RenderingHints(JAI.KEY_IMAGE_LAYOUT, il);
        // Convert the data to byte.
        ParameterBlock pb = new ParameterBlock();
        pb.addSource(idft);
        pb.add(DataBuffer.TYPE_BYTE);
        PlanarImage dst = JAI.create("format", pb, rh);
        return dst;
    }

    private void display(PlanarImage dst) {
        // Display.
        addComponent(new ScrollingAwtImagePanel(dst, dst.getWidth(), dst.getHeight()));
        pack();
        setVisible(true);
    }


    public RenderedOp DFTImage(PlanarImage image) {
        ParameterBlockJAI pb = new ParameterBlockJAI("dft");
        pb.addSource(image);
        pb.setParameter("scalingType", DFTDescriptor.SCALING_NONE);
        pb.setParameter("dataNature", DFTDescriptor.REAL_TO_COMPLEX);
        return JAI.create("dft", pb);
    }

    public PlanarImage inverseDFTImage(RenderedOp image) {
        ParameterBlockJAI pb = new ParameterBlockJAI("idft");
        pb.addSource(image);
        pb.setParameter("scalingType", DFTDescriptor.SCALING_NONE);
        pb.setParameter("dataNature", DFTDescriptor.COMPLEX_TO_REAL);
        return JAI.create("idft", pb);
    }

    public RenderedOp multComplex(RenderedOp img1, RenderedOp img2) {
        ParameterBlockJAI pb = new ParameterBlockJAI("multiplycomplex");
        pb.addSource(img1);
        pb.addSource(img2);
        return JAI.create("multiplycomplex", pb);
    }

    public RenderedOp magnitudeImage(RenderedOp image) {
        ParameterBlockJAI pb = new ParameterBlockJAI("magnitude");
        pb.addSource(image);
        return JAI.create("magnitude", pb);
    }

    public RenderedOp phaseImage(RenderedOp image) {
        ParameterBlockJAI pb = new ParameterBlockJAI("phase");
        pb.addSource(image);
        return JAI.create("phase", pb);
    }

    public RenderedOp polarToComplexImage(RenderedOp mag,
                                          RenderedOp phase) {
        ParameterBlockJAI pb = new ParameterBlockJAI("polartocomplex");
        pb.addSource(mag);
        pb.addSource(phase);
        return JAI.create("polartocomplex", pb);
    }

    public static void testDft() {
        ClosableJFrame cf = new ClosableJFrame();
        Container c = cf.getContentPane();
        int SIZE = 18;
//        // Create a constant SIZE x SIZE byte image of unity pixel values.
        ParameterBlock pb = new ParameterBlock();
        pb.add((float) SIZE).add((float) SIZE);
        pb.add(new Byte[]{new Byte((byte) 1)});
//        RenderedImage constant = JAI.create("constant", pb);
        RenderedImage constant = ImageUtils.getPlanarImage(ImageUtils.getImage());

        // Create a TiledImage of the same size but set only its central
        // quarter to the constant image to yield a rectangle of unity pixel
        // values in a black background.
        TiledImage source = new TiledImage(constant.getMinX(),
                constant.getMinY(),
                constant.getWidth(),
                constant.getHeight(),
                constant.getTileGridXOffset(),
                constant.getTileGridYOffset(),
                constant.getSampleModel(),
                constant.getColorModel());
        ROI roi = new ROIShape(new Rectangle(SIZE / 3, SIZE / 3, SIZE / 3, SIZE / 3));
        source.set(constant, roi);

        // Create RenderingHints such that the "dft" output is double
        // precision (default is single precision float).
        SampleModel sm =
                RasterFactory.createComponentSampleModel(source.getSampleModel(),
                        DataBuffer.TYPE_DOUBLE,
                        SIZE, SIZE, 1);
        ImageLayout il = new ImageLayout();
        il.setSampleModel(sm);
        RenderingHints hints = new RenderingHints(JAI.KEY_IMAGE_LAYOUT, il);

        // Perform forward then inverse DFTs and extract the first band
        // from the inverse transform.
        pb = (new ParameterBlock()).addSource(source);
        RenderedImage dft = JAI.create("dft", pb, hints);
        RenderedImage idft = JAI.create("idft", dft);
        RenderedImage band1 = JAI.create("bandselect", idft, new int[]{0});

        // Subtract the first band of the inverse transform from the original
        // source TiledImage, take the absolute value of the difference and
        // calculate the absolute deviation.
        RenderedImage diff = JAI.create("subtract", band1, source);
        RenderedImage abs = JAI.create("absolute", diff);
        RenderedImage ext = JAI.create("extrema", abs);

        // Print the absolute deviation.
        double[] absdev = (double[]) ext.getProperty("maximum");
        for (int b = 0; b < absdev.length; b++) {
            System.out.println("Band " + b + " absolute deviation = " + absdev[b]);
        }

        // Rescale the first band of the inverse transform for display.
        RenderedImage rescale = JAI.create("rescale", band1,
                new double[]{255.0},
                new double[]{0.0});
        RenderedImage display = JAI.create("format", rescale,
                DataBuffer.TYPE_BYTE);

        // Display the original source and the first band of the inverse DFT.
        c.setLayout(new GridLayout(1, 2));
        c.add(new j2d.widgets.ScrollingImagePanel(JAI.create("rescale", source,
                new double[]{255.0},
                new double[]{0.0}),
                SIZE, SIZE));
        c.add(new ScrollingImagePanel(display, SIZE, SIZE));
        cf.pack();
        cf.setVisible(true);
    }

}